@@ -1,11 +1,13 @@
+# frozen_string_literal: true
+
 module OroGen
     module Loaders
-        @macos =  RbConfig::CONFIG["host_os"] =~%r!([Dd]arwin)!
+        @macos = RbConfig::CONFIG['host_os'] =~ /[Dd]arwin/
         def self.macos?
             @macos
         end
 
-        @windows = RbConfig::CONFIG["host_os"] =~%r!(msdos|mswin|djgpp|mingw|[Ww]indows)!
+        @windows = RbConfig::CONFIG['host_os'] =~ /msdos|mswin|djgpp|mingw|[Ww]indows/
         def self.windows?
             @windows
         end
@@ -32,7 +34,9 @@ module OroGen
             TaskLibrary = Struct.new :pkg
             Type = Struct.new :name, :exported
 
-            def to_s; "#<OroGen::Loaders::PkgConfig(#{object_id.to_s(16)}) #{orocos_target}>" end
+            def to_s
+                "#<OroGen::Loaders::PkgConfig(#{object_id.to_s(16)}) #{orocos_target}>"
+            end
 
             # @return [String] the name of the orocos target we are looking for
             attr_reader :orocos_target
@@ -68,11 +72,11 @@ module OroGen
 
             def clear
                 super
-                @available_projects = Hash.new
-                @available_typekits = Hash.new
-                @available_task_models = Hash.new
-                @available_deployments = Hash.new
-                @available_deployed_tasks = Hash.new
+                @available_projects = {}
+                @available_typekits = {}
+                @available_task_models = {}
+                @available_deployments = {}
+                @available_deployed_tasks = {}
                 @available_types = nil
             end
 
@@ -81,22 +85,21 @@ module OroGen
             # @param [Utilrb::PkgConfig] the pkg-config from the project
             #   (orogen-project-NAME)
             # @return [Project] the registered project
-            def add_project_from(pkg)
-            end
-            
+            def add_project_from(pkg); end
+
             def find_pkgconfig(name, minimal: true)
                 Utilrb::PkgConfig.get(name, minimal: minimal)
-            rescue Utilrb::PkgConfig::NotFound
+            rescue Utilrb::PkgConfig::NotFound # rubocop:disable Lint/HandleExceptions
             end
 
             def has_pkgconfig?(name)
-                !!find_pkgconfig(name, minimal: true)
+                find_pkgconfig(name, minimal: true)
             end
 
             def has_project?(name)
                 if super
                     true
-                elsif available_projects.has_key?(name)
+                elsif available_projects.key?(name)
                     available_projects[name]
                 else
                     available_projects[name] = has_pkgconfig?("orogen-project-#{name}")
@@ -106,14 +109,14 @@ module OroGen
             def has_typekit?(name)
                 if super
                     true
-                elsif available_typekits.has_key?(name)
+                elsif available_typekits.key?(name)
                     available_typekits[name]
                 elsif !has_pkgconfig?("#{name}-typekit-#{orocos_target}")
                     available_typekits[name] = false
                 elsif !(project_pkg = find_pkgconfig("orogen-project-#{name}"))
                     available_typekits[name] = false
                 else
-                    available_typekits[name] = !!project_pkg.type_registry
+                    available_typekits[name] = project_pkg.type_registry
                 end
             end
 
@@ -132,7 +135,7 @@ module OroGen
                 elsif pkg.deffile.empty?
                     warn "#{pkg.name}.pc does not have a deffile field"
                 end
-                return File.read(pkg.deffile), pkg.deffile
+                [File.read(pkg.deffile), pkg.deffile]
             end
 
             def typekit_model_text_from_name(name)
@@ -147,11 +150,11 @@ module OroGen
                 registry = File.read(pkg.type_registry)
                 typelist = File.join(File.dirname(pkg.type_registry), "#{name}.typelist")
                 typelist = File.read(typelist)
-                return registry, typelist
+                [registry, typelist]
             end
 
             def find_task_library_from_task_model_name(model_name)
-                if available_task_models.has_key?(model_name)
+                if available_task_models.key?(model_name)
                     return available_task_models[model_name]
                 end
 
@@ -164,7 +167,7 @@ module OroGen
                     end
                 end
 
-                if available_task_models.has_key?(model_name)
+                if available_task_models.key?(model_name)
                     available_task_models[model_name]
                 else
                     available_task_models[model_name] = nil
@@ -179,7 +182,8 @@ module OroGen
             def resolve_task_library_package(name, minimal: false)
                 Utilrb::PkgConfig.get("#{name}-tasks-#{orocos_target}", minimal: minimal)
             rescue Utilrb::PkgConfig::NotFound
-                raise TaskLibraryNotFound, "the '#{name}' typekit is not available to pkgconfig"
+                raise TaskLibraryNotFound,
+                      "the '#{name}' typekit is not available to pkgconfig"
             end
 
             # Return the pkgconfig description for a given typekit package
@@ -188,9 +192,12 @@ module OroGen
             # @return [Utilrb::PkgConfig] the package
             # @raise TypekitNotFound if the package cannot be found
             def resolve_typekit_package(name, minimal: false)
-                Utilrb::PkgConfig.get("#{name}-typekit-#{orocos_target}", minimal: minimal)
+                Utilrb::PkgConfig.get(
+                    "#{name}-typekit-#{orocos_target}", minimal: minimal
+                )
             rescue Utilrb::PkgConfig::NotFound
-                raise TypekitNotFound, "the '#{name}' typekit is not available to pkgconfig"
+                raise TypekitNotFound,
+                      "the '#{name}' typekit is not available to pkgconfig"
             end
 
             # Return the pkgconfig description for a given transport package
@@ -199,13 +206,19 @@ module OroGen
             # @return [Utilrb::PkgConfig] the package
             # @raise TransportNotFound if the package cannot be found
             def resolve_transport_package(typekit_name, transport_name, minimal: false)
-                Utilrb::PkgConfig.get("#{typekit_name}-transport-#{transport_name}-#{orocos_target}", minimal: minimal)
+                Utilrb::PkgConfig.get(
+                    "#{typekit_name}-transport-#{transport_name}-"\
+                    "#{orocos_target}",
+                    minimal: minimal
+                )
             rescue Utilrb::PkgConfig::NotFound
-                raise TransportNotFound, "the '#{transport_name}' transport for the '#{typekit_name}' typekit is not available to pkgconfig"
+                raise TransportNotFound,
+                      "the '#{transport_name}' transport for the '#{typekit_name}' "\
+                      'typekit is not available to pkgconfig'
             end
 
             # @api private
-            # 
+            #
             # Resolve the full path to a library from a pkg-config package
             #
             # @param [Utilrb::PkgConfig] package the package whose library dirs
@@ -245,13 +258,15 @@ module OroGen
             DeploymentInfo = Struct.new :project_name, :binfile
 
             def find_project_from_deployment_name(deployment_name)
-                if available_deployments.has_key?(deployment_name)
+                if available_deployments.key?(deployment_name)
                     return available_deployments[deployment_name].project_name
                 end
-                
-                if deployment_pkg = find_pkgconfig("orogen-#{deployment_name}")
+
+                if (deployment_pkg = find_pkgconfig("orogen-#{deployment_name}"))
                     if has_project?(deployment_pkg.project_name)
-                        info = DeploymentInfo.new(deployment_pkg.project_name, deployment_pkg.binfile)
+                        info = DeploymentInfo.new(
+                            deployment_pkg.project_name, deployment_pkg.binfile
+                        )
                         available_deployments[deployment_name] = info
                         info.project_name
                     else
@@ -269,68 +284,71 @@ module OroGen
             end
 
             def load_available_deployed_tasks
-                @available_deployed_tasks = Hash.new
+                @available_deployed_tasks = {}
 
                 Utilrb::PkgConfig.each_package(/^orogen-\w+$/) do |pkg_name|
                     pkg = Utilrb::PkgConfig.get(pkg_name, minimal: true)
                     project_name = pkg.project_name
                     deployment_name = pkg_name.gsub(/^orogen-/, '')
 
-                    if !pkg.deployed_tasks_with_models
-                        # oroGen has a bug, in which it installed the pkg-config
-                        # file for deployments that were not meant to be
-                        # installed.
-                        #
-                        # Workaround
-                        next
-                    elsif !has_project?(project_name)
-                        next
-                    end
+                    # oroGen has a bug, in which it installed the pkg-config
+                    # file for deployments that were not meant to be
+                    # installed.
+                    #
+                    # Workaround
+                    next unless pkg.deployed_tasks_with_models
+                    next unless has_project?(project_name)
 
-                    pkg.deployed_tasks_with_models.split(',').each_slice(2) do |task_name, task_model_name|
-                        available_deployed_tasks[task_name] ||= Set.new
-                        available_deployed_tasks[task_name] <<
-                            AvailableDeployedTask.new(
-                                task_name, deployment_name,
-                                task_model_name, project_name)
-                        available_deployments[deployment_name] =
-                            DeploymentInfo.new(project_name, pkg.binfile)
-                    end
+                    pkg.deployed_tasks_with_models.split(',')
+                       .each_slice(2) do |task_name, task_model_name|
+                           available_deployed_tasks[task_name] ||= Set.new
+                           available_deployed_tasks[task_name] <<
+                               AvailableDeployedTask.new(
+                                   task_name, deployment_name,
+                                   task_model_name, project_name
+                               )
+                           available_deployments[deployment_name] =
+                               DeploymentInfo.new(project_name, pkg.binfile)
+                       end
                 end
             end
 
             def find_deployments_from_deployed_task_name(name)
-                if available_deployed_tasks.empty?
-                    load_available_deployed_tasks
-                end
+                load_available_deployed_tasks if available_deployed_tasks.empty?
 
-                if available_deployed_tasks.has_key?(name)
-                    return available_deployed_tasks[name].map(&:deployment_name)
-                else return Set.new
+                if available_deployed_tasks.key?(name)
+                    available_deployed_tasks[name].map(&:deployment_name)
+                else
+                    Set.new
                 end
             end
 
             def load_available_types
-                @available_types = Hash.new
+                @available_types = {}
 
                 Utilrb::PkgConfig.each_package(/-typekit-#{orocos_target}$/) do |pkg_name|
                     pkg = Utilrb::PkgConfig.get(pkg_name, minimal: true)
-                    next if !pkg.type_registry
+                    next unless pkg.type_registry
 
                     typelist = pkg.type_registry.gsub(/tlb$/, 'typelist')
                     typelist, typelist_exported =
                         OroGen::Spec::Typekit.parse_typelist(File.read(typelist))
-                    typelist = typelist - typelist_exported
+                    typelist -= typelist_exported
                     typelist.compact.each do |typename|
-                        if existing = available_types[typename]
-                            OroGen.info "#{typename} is defined by both #{existing[0]} and #{pkg.project_name}, registering #{existing[0]} as the responsible typekit"
+                        if (existing = available_types[typename])
+                            OroGen.info "#{typename} is defined by both #{existing[0]} "\
+                                        "and #{pkg.project_name}, registering "\
+                                        "#{existing[0]} as the responsible typekit"
                         else
                             available_types[typename] = Type.new(pkg.project_name, false)
                         end
                     end
                     typelist_exported.compact.each do |typename|
-                        if existing = available_types[typename]
-                            OroGen.info "#{typename} is defined by both #{existing[0]} and #{pkg.project_name}, registering #{pkg.project_name} as it exports the type but #{existing[0]} does not"
+                        if (existing = available_types[typename])
+                            OroGen.info "#{typename} is defined by both #{existing[0]} "\
+                                        "and #{pkg.project_name}, registering "\
+                                        "#{pkg.project_name} as it exports the type "\
+                                        "but #{existing[0]} does not"
                         end
                         available_types[typename] = Type.new(pkg.project_name, true)
                     end
@@ -350,27 +368,29 @@ module OroGen
                            else type
                            end
 
-                if typekits = typekits_by_type_name[typename]
-                    if export_tk = typekits.find { |tk| tk.interface_type?(typename) }
-                        return export_tk
-                    elsif !exported
-                        return typekits.first
-                    end
+                if (typekits = typekits_by_type_name[typename])
+                    export_tk = typekits.find { |tk| tk.interface_type?(typename) }
+                    return export_tk if export_tk
+                    return typekits.first unless exported
                 end
 
-                if !available_types
-                    load_available_types
-                end
+                load_available_types unless available_types
 
                 typekit = available_types[typename]
                 if !typekit
-                    raise NotTypekitType.new(typename), "no type #{typename} has been registered in an oroGen project"
+                    raise NotTypekitType.new(typename),
+                          "no type #{typename} has been registered in an oroGen project"
                 elsif exported && !typekit.exported
                     begin
                         typekits = imported_typekits_for(typename)
-                        raise NotExportedType.new(typename, typekits), "the type #{typename} is registered in the #{typekits.map(&:name).sort.join(", ")} typekit, but it is not exported to the RTT type system"
+                        raise NotExportedType.new(typename, typekits),
+                              "the type #{typename} is registered in the "\
+                              "#{typekits.map(&:name).sort.join(', ')} typekit, "\
+                              'but it is not exported to the RTT type system'
                     rescue DefinitionTypekitNotFound
-                        raise NotExportedType.new(typename, []), "the type #{typename} is not exported to the RTT type system"
+                        raise NotExportedType.new(typename, []),
+                              "the type #{typename} is not exported to the RTT "\
+                              'type system'
                     end
                 else
                     typekit_model_from_name(typekit.name)
@@ -378,26 +398,31 @@ module OroGen
             end
 
             def each_available_project_name
-                return enum_for(__method__) if !block_given?
+                return enum_for(__method__) unless block_given?
+
                 rx = /^orogen-project-/
                 Utilrb::PkgConfig.each_package(rx) do |pkg_name|
                     yield(pkg_name.gsub(rx, ''))
                 end
             end
 
-            def each_available_typekit_name(&block)
-                return enum_for(__method__) if !block_given?
+            def each_available_typekit_name
+                return enum_for(__method__) unless block_given?
+
                 rx = /-typekit-#{orocos_target}$/
                 Utilrb::PkgConfig.each_package(rx) do |pkg_name|
                     yield(pkg_name.gsub(rx, ''))
                 end
             end
 
-            def each_available_task_model_name(&block)
-                return enum_for(__method__) if !block_given?
+            def each_available_task_model_name
+                return enum_for(__method__) unless block_given?
+
                 each_available_project_name do |project_name|
                     begin
-                        pkg = Utilrb::PkgConfig.new("#{project_name}-tasks-#{orocos_target}")
+                        pkg = Utilrb::PkgConfig.new(
+                            "#{project_name}-tasks-#{orocos_target}"
+                        )
                     rescue Utilrb::PkgConfig::NotFound
                         next
                     end
@@ -407,8 +432,9 @@ module OroGen
                 end
             end
 
-            def each_available_deployment_name(&block)
-                return enum_for(__method__) if !block_given?
+            def each_available_deployment_name
+                return enum_for(__method__) unless block_given?
+
                 rx = /^orogen-\w+$/
                 Utilrb::PkgConfig.each_package(rx) do |pkg_name|
                     yield(pkg_name.gsub(/^orogen-/, ''))
@@ -416,8 +442,8 @@ module OroGen
             end
 
             # @deprecated use {#each_available_deployed_task} instead
-            def each_available_deployed_task_name(&block)
-                return enum_for(__method__) if !block_given?
+            def each_available_deployed_task_name
+                return enum_for(__method__) unless block_given?
 
                 each_available_deployed_task do |task|
                     yield(task.task_name, task.deployment_name)
@@ -432,14 +458,12 @@ module OroGen
             # system
             #
             # @yieldparam [AvailableDeployedTask]
-            def each_available_deployed_task(&block)
-                return enum_for(__method__) if !block_given?
+            def each_available_deployed_task
+                return enum_for(__method__) unless block_given?
 
-                if available_deployed_tasks.empty?
-                    load_available_deployed_tasks
-                end
+                load_available_deployed_tasks if available_deployed_tasks.empty?
 
-                available_deployed_tasks.each do |deployed_task_name, deployments|
+                available_deployed_tasks.each_value do |deployments|
                     deployments.each do |task|
                         yield(task)
                     end
@@ -451,9 +475,7 @@ module OroGen
             # @param [String] task_name
             # @return [AvailableDeployedTask,nil]
             def find_basic_deployed_task_info(task_name)
-                if available_deployed_tasks.empty?
-                    load_available_deployed_tasks
-                end
+                load_available_deployed_tasks if available_deployed_tasks.empty?
                 available_deployed_tasks[task_name]
             end
 
@@ -464,12 +486,10 @@ module OroGen
             #   defines it
             # @yieldparam [Boolean] exported whether this type is exported or
             #   not
-            def each_available_type_name(&block)
-                return enum_for(__method__) if !block_given?
+            def each_available_type_name
+                return enum_for(__method__) unless block_given?
 
-                if !available_types
-                    load_available_types
-                end
+                load_available_types unless available_types
                 available_types.each do |type_name, type_info|
                     yield(type_name, type_info.name, type_info.exported)
                 end
