@@ -46,16 +46,19 @@ module OroGen
             #
             # @return [Project]
             attr_reader :project
+
             # The loader that has been used to load this task context
             #
             # @return [Loaders::Base]
-            def loader; project.loader end
+            def loader
+                project.loader
+            end
 
             ## :method: doc
             # :call-seq:
             #   doc => string
             #   doc "documentation string"
-            #   
+            #
             # Gets or sets the documentation string for this task context
             dsl_attribute :doc
 
@@ -104,7 +107,9 @@ module OroGen
             attr_reader :extensions
 
             # True if an extension with the given name has been registered
-            def has_extension?(name, with_subclasses = true); !!find_extension(name, with_subclasses) end
+            def has_extension?(name, with_subclasses = true)
+                find_extension(name, with_subclasses)
+            end
 
             # Registers an extension with the given name. Raises ArgumentError
             # if there is already one.
@@ -119,16 +124,14 @@ module OroGen
 
             # Enumerates the extensions registered on this task model, as (name,
             # extension_object) pairs
-            def each_extension(with_subclasses = true, &block)
-                if !block_given?
-                    return enum_for(:each_extension, with_subclasses)
-                end
+            def each_extension(with_subclasses = true)
+                return enum_for(:each_extension, with_subclasses) unless block_given?
 
                 seen = Set.new
                 klass = self
                 begin
                     klass.extensions.each do |ext|
-                        if !seen.include?(ext.name)
+                        unless seen.include?(ext.name)
                             seen << ext.name
                             yield(ext)
                         end
@@ -149,14 +152,19 @@ module OroGen
             # Returns the extension named +name+, or raises ArgumentError if
             # none is registered with that name
             def extension(name, with_subclasses = true)
-                if ext = find_extension(name, with_subclasses)
+                if (ext = find_extension(name, with_subclasses))
                     ext
                 else raise ArgumentError, "no extension registered under the name '#{name}'"
                 end
             end
 
-            def to_s; "#<OroGen::Spec::TaskContext: #{name}>" end
-            def inspect; to_s end
+            def to_s
+                "#<OroGen::Spec::TaskContext: #{name}>"
+            end
+
+            def inspect
+                to_s
+            end
 
             # The task name
             attr_reader :name
@@ -173,13 +181,55 @@ module OroGen
 
             # Call to declare that this task model is not meant to run in
             # practice
-            def abstract; @abstract = true; end
+            def abstract
+                @abstract = true
+            end
+
             # True if this task model is only meant to declare an interface, and
             # should not be deployed
-            def abstract?; @abstract end
+            def abstract?
+                @abstract
+            end
 
-            def use_qt; @uses_qt = true; end
-            def uses_qt?; @uses_qt; end
+            def use_qt
+                needs_global_initializer(:qt)
+            end
+
+            def uses_qt?
+                needs_global_initializer?(:qt)
+            end
+
+            # Specifies that deployments that use this task must run a global
+            # initializer in the global main
+            #
+            # @param [Symbol] key the initializer name. It must be previously
+            #   registered with {Spec::Deployment#register_global_initializer}.
+            def needs_global_initializer(key)
+                unless Spec::Deployment.has_global_initializer?(key)
+                    raise ArgumentError, "unknown global initializer '#{key}'"
+                end
+
+                @global_initializers << key
+            end
+
+            def needs_global_initializer?(key)
+                @global_initializers.include?(key) ||
+                    superclass&.needs_global_initializer?(key)
+            end
+
+            # Enumerates the global initializers this task required
+            #
+            # @see needs_global_initializer
+            def each_needed_global_initializer
+                return enum_for(__method__) unless block_given?
+
+                @global_initializers.each { |i| yield(i) }
+                unique = @global_initializers.dup
+
+                superclass&.each_needed_global_initializer do |i|
+                    yield(i) unless unique.include?(i)
+                end
+            end
 
             # Declares that this task context is a subclass of the following
             # TaskContext class. +task_context+ can either be a class name or a
@@ -306,7 +356,7 @@ module OroGen
             def periodic(period)
                 default_activity :periodic, period
             end
-            
+
             # True if this task context is defined by one of our dependencies.
             attr_predicate :external_definition?, true
 
@@ -325,7 +375,7 @@ module OroGen
             # TaskContext objects should not be created directly. You should
             # use {Project#task_context} for that.
             def initialize(project, name = nil, subclasses: project.default_task_superclass)
-                @project  = project
+                @project = project
 
                 if subclasses
                     @superclass =
@@ -344,26 +394,28 @@ module OroGen
 
                 @implemented_classes = []
                 @name = name
+                @abstract = false
 
                 # This is an array, as we don't want to have it reordered
                 # unnecessarily
-                @states = Array.new
+                @states = []
 
-                @properties = Hash.new
-                @attributes = Hash.new
-                @operations = Hash.new
-                @output_ports = Hash.new
-                @input_ports  = Hash.new
-                @dynamic_ports = Array.new
-                @event_ports = Hash.new
+                @properties = {}
+                @attributes = {}
+                @operations = {}
+                @output_ports = {}
+                @input_ports  = {}
+                @dynamic_ports = []
+                @event_ports = {}
                 @initial_state = 'Stopped'
-                @default_extensions = Array.new
+                @default_extensions = []
                 @fixed_initial_state = false
                 @needs_configuration = false
+                @global_initializers = Set.new
 
                 ## WARN: this must be kept an array so that the generation order
                 ## WARN: is deterministic
-                @extensions = Array.new
+                @extensions = []
 
                 super()
 
@@ -448,7 +500,7 @@ module OroGen
             end
 
             # Raises ArgumentError if an object named +name+ is already present
-            # in the set attribute +set_name+. 
+            # in the set attribute +set_name+.
             #
             # This is an internal helper method
             def check_uniqueness(name) # :nodoc:
@@ -613,10 +665,10 @@ module OroGen
                 state_port = find_port("state")
                 if state_port
                     if state_port.kind_of?(InputPort)
-                        raise ArgumentError, 
+                        raise ArgumentError,
                             "there is already an input port called 'state', cannot enable extended state support"
                     elsif state_port.type != project.find_type("/int32_t")
-                        raise ArgumentError, 
+                        raise ArgumentError,
                             "there is already an output port called 'state', but it is not of type 'int' (found #{state_port.type_name}"
                     end
                 else
@@ -1182,7 +1234,7 @@ module OroGen
                 end
                 return false
             end
-            
+
             # Return true if this task interface has an dynamic property.
             def has_dynamic_attributes?
                 self_attributes.each do |p|
@@ -1330,7 +1382,7 @@ module OroGen
                     label << "  </TD></TR>\n"
                     label << "</TABLE>"
                 end
-                    
+
                 result = ""
                 result << "  node [shape=none,margin=0,height=.1];"
 
